package sun.tracing;

import java.lang.annotation.Annotation;
import java.lang.reflect.AnnotatedElement;
import com.sun.tracing.ProviderName;
import com.sun.tracing.Probe;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Proxy;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.lang.reflect.Method;
import java.util.HashMap;
import com.sun.tracing.Provider;
import java.lang.reflect.InvocationHandler;

public abstract class ProviderSkeleton implements InvocationHandler, Provider
{
    protected boolean active;
    protected Class<? extends Provider> providerType;
    protected HashMap<Method, ProbeSkeleton> probes;
    
    protected abstract ProbeSkeleton createProbe(Method p0);
    
    protected ProviderSkeleton(Class<? extends Provider> providerType) {
        this.active = false;
        this.providerType = providerType;
        this.probes = new HashMap();
    }
    
    public void init() {
        Method[] array = (Method[])AccessController.doPrivileged(new PrivilegedAction<Method[]>() {
            @Override
            public Method[] run() {
                return ProviderSkeleton.this.providerType.getDeclaredMethods();
            }
        });
        int length = array.length;
        int i = 0;
        while (i < length) {
            Method key = array[i];
            if (key.getReturnType() != Void.TYPE) {
                throw new IllegalArgumentException("Return value of method is not void");
            }
            this.probes.put(key, this.createProbe(key));
            ++i;
        }
        this.active = true;
    }
    
    public <T extends Provider> T newProxyInstance() {
        return (T)AccessController.doPrivileged(new PrivilegedAction<T>() {
            
            @Override
            public T run() {
                return (Provider)Proxy.newProxyInstance(ProviderSkeleton.this.providerType.getClassLoader(), new Class[] { ProviderSkeleton.this.providerType }, ih);
            }
        });
    }
    
    @Override
    public Object invoke(Object o, Method method, Object[] args) {
        Class<Object> declaringClass = method.getDeclaringClass();
        if (declaringClass != this.providerType) {
            try {
                if (declaringClass == Provider.class || declaringClass == Object.class) {
                    return method.invoke(this, args);
                }
                throw new SecurityException();
            }
            catch (IllegalAccessException ex) {
                assert false;
            }
            catch (InvocationTargetException ex2) {
                assert false;
            }
        }
        else {
            this.triggerProbe(method, args);
        }
        return null;
    }
    
    public Probe getProbe(Method key) {
        return this.active ? ((Probe)this.probes.get(key)) : null;
    }
    
    public void dispose() {
        this.active = false;
        this.probes.clear();
    }
    
    protected String getProviderName() {
        return getAnnotationString(this.providerType, ProviderName.class, this.providerType.getSimpleName());
    }
    
    protected static String getAnnotationString(AnnotatedElement annotatedElement, Class<? extends Annotation> clazz, String s) {
        String s2 = (String)getAnnotationValue(annotatedElement, clazz, "value", s);
        return s2.isEmpty() ? s : s2;
    }
    
    protected static Object getAnnotationValue(AnnotatedElement annotatedElement, Class<? extends Annotation> clazz, String name, Object invoke) {
        try {
            invoke = clazz.getMethod(name, new Class[0]).invoke(annotatedElement.getAnnotation(clazz), new Object[0]);
        }
        catch (NoSuchMethodException ex) {
            assert false;
        }
        catch (IllegalAccessException ex2) {
            assert false;
        }
        catch (InvocationTargetException ex3) {
            assert false;
        }
        catch (NullPointerException ex4) {
            assert false;
        }
        return invoke;
    }
    
    protected void triggerProbe(Method key, Object[] array) {
        if (this.active) {
            ProbeSkeleton probeSkeleton = (ProbeSkeleton)this.probes.get(key);
            if (probeSkeleton != null) {
                probeSkeleton.uncheckedTrigger(array);
            }
        }
    }
}
